@@ -3,7 +3,7 @@ package Crypt::JWT;
 use strict;
 use warnings;
 
-our $VERSION = '0.008';
+our $VERSION = '0.009';
 
 use Exporter 'import';
 our %EXPORT_TAGS = ( all => [qw(decode_jwt encode_jwt)] );
@@ -113,14 +113,14 @@ sub _verify_claims {
   my $leeway = $args{leeway} || 0;
   my $now = time;
 
-  ### iat
-  if(defined $payload->{iat}) {
-    if (!defined $args{verify_iat} || $args{verify_iat}==1) {
-      croak "JWT: iat claim check failed ($payload->{iat}/$leeway vs. $now)" if $payload->{iat} - $leeway > $now;
+  ### exp
+  if(defined $payload->{exp}) {
+    if (!defined $args{verify_exp} || $args{verify_exp}==1) {
+      croak "JWT: exp claim check failed ($payload->{exp}/$leeway vs. $now)" if $payload->{exp} + $leeway < $now;
     }
   }
-  elsif ($args{verify_iat} && $args{verify_iat}==1) {
-    croak "JWT: iat claim required but missing"
+  elsif ($args{verify_exp} && $args{verify_exp}==1) {
+    croak "JWT: exp claim required but missing"
   }
 
   ### nbf
@@ -133,14 +133,16 @@ sub _verify_claims {
     croak "JWT: nbf claim required but missing"
   }
 
-  ### exp
-  if(defined $payload->{exp}) {
-    if (!defined $args{verify_exp} || $args{verify_exp}==1) {
-      croak "JWT: exp claim check failed ($payload->{exp}/$leeway vs. $now)" if $payload->{exp} + $leeway < $now;
+  ### iat
+  if (exists $args{verify_iat}) { #default (non existing verify_iat) == no iat check
+    if(defined $payload->{iat}) {
+      if (!defined $args{verify_iat} || $args{verify_iat}==1) {
+        croak "JWT: iat claim check failed ($payload->{iat}/$leeway vs. $now)" if $payload->{iat} - $leeway > $now;
+      }
+    }
+    elsif ($args{verify_iat} && $args{verify_iat}==1) {
+      croak "JWT: iat claim required but missing"
     }
-  }
-  elsif ($args{verify_exp} && $args{verify_exp}==1) {
-    croak "JWT: exp claim required but missing"
   }
 
   ### iss
@@ -411,8 +413,9 @@ sub _encode_jwe {
   # prepare header
   $header->{alg} = $alg;
   $header->{enc} = $enc;
-  $header->{typ} = 'JWE' if $args{auto_typ};
+  #REMOVED: $header->{typ} = 'JWT' if !exists $header->{typ} && $args{auto_typ};
   # key
+  croak "JWE: missing 'key'" if !$args{key};
   my $key = defined $args{keypass} ? [$args{key}, $args{keypass}] : $args{key};
   # prepare cek
   my ($cek, $ecek) = _encrypt_jwe_cek($key, $header); # adds some header items
@@ -461,6 +464,7 @@ sub _decode_jwe {
     croak "JWT: enc '$header->{enc}' not in accepted_enc" if !$acce{$header->{enc}};
   }
 
+  croak "JWE: missing 'key'" if !$key;
   my $cek = _decrypt_jwe_cek($ecek, $key, $header);
   my $payload = _decrypt_jwe_payload($cek, $header->{enc}, $b64u_header, $ct, $iv, $tag);
   $payload = _payload_unzip($payload, $header->{zip}) if $header->{zip};
@@ -542,11 +546,12 @@ sub _encode_jws {
   my $b64u_payload = encode_base64url($payload);
   # prepare header
   $header->{alg} = $alg;
-  $header->{typ} = 'JWS' if $args{auto_typ};
+  #REMOVED: $header->{typ} = 'JWT' if !exists $header->{typ} && $args{auto_typ};
   # encode header
   my $json_header = encode_json($header);
   my $b64u_header = encode_base64url($json_header);
   # key
+  croak "JWS: missing 'key'" if !$args{key} && $alg ne 'none';
   my $key = defined $args{keypass} ? [$args{key}, $args{keypass}] : $args{key};
   # sign header
   my $b64u_signature = _sign_jws($b64u_header, $b64u_payload, $alg, $key);
@@ -575,6 +580,7 @@ sub _decode_jws {
       my $k = _kid_lookup($header->{kid}, $args{kid_keys});
       $key = $k if defined $k;
     }
+    croak "JWS: missing 'key'" if !$key && $alg ne 'none';
     my $valid = _verify_jws($b64u_header, $b64u_payload, $b64u_sig, $alg, $key);
     croak "JWS: decode failed" if !$valid;
   }
@@ -653,8 +659,6 @@ Crypt::JWT - JSON Web Token (JWT, JWS, JWE) as defined by RFC7519, RFC7515, RFC7
 
 =head1 DESCRIPTION
 
-B<BEWARE:> experimental, unfinished, unstable, work in progress!
-
 Implements B<JSON Web Token (JWT)> - L<https://tools.ietf.org/html/rfc7519>.
 The implementation covers not only B<JSON Web Signature (JWS)> - L<https://tools.ietf.org/html/rfc7515>,
 but also B<JSON Web Encryption (JWE)> - L<https://tools.ietf.org/html/rfc7516>.
@@ -747,11 +751,10 @@ When 'key' parameter is an encrypted private RSA or ECC key this optional parame
 
 This parametes can be either a JWK Set JSON string (see RFC7517) or a perl HASH ref with JWK Set structure like this:
 
-  my $keylist => {
+  my $keylist = {
     keys => [
-      { kid=>"key1", kty=>"oct", "k":"GawgguFyGrWKav7AX4VKUg" },
-      { kid=>"key2", kty=>"oct", "k":"GawgguFyGrWKav7AX4VKUg" },
-      { kid=>"key3", kty=>"EC", crv=>"P-256", x=>"f83OJ..", y=>"x_FEz.." },
+      { kid=>"key1", kty=>"oct", k=>"GawgguFyGrWKav7AX4VKUg" },
+      { kid=>"key2", kty=>"oct", k=>"ulxLGy4XqhbpkR5ObGh1gX" },
     ]
   };
   my $payload = decode_jwt(token=>$t, kid_keys=>$keylist);
@@ -855,31 +858,31 @@ C<undef> (default) - do not verify 'jti' claim
 
 =item verify_iat
 
-C<0> - ignore 'iat' claim
+C<undef> - Issued At 'iat' claim must be valid (not in the future) if present
+
+C<0> (default) - ignore 'iat' claim
 
 C<1> - require valid 'iat' claim
 
-C<undef> (default) - 'iat' claim must be valid if present
-
 =item verify_nbf
 
+C<undef> (default) - Not Before 'nbf' claim must be valid if present
+
 C<0> - ignore 'nbf' claim
 
 C<1> - require valid 'nbf' claim
 
-C<undef> (default) - 'nbf' claim must be valid if present
-
 =item verify_exp
 
+C<undef> (default) - Expiration Time 'exp' claim must be valid if present
+
 C<0> - ignore 'exp' claim
 
 C<1> - require valid 'exp' claim
 
-C<undef> (default) - 'exp' claim must be valid if present
-
 =item leeway
 
-Tolerance in seconds related to C<verify_exp>, C<verify_nbf>, C<verify_iat>. Default is C<0>.
+Tolerance in seconds related to C<verify_exp>, C<verify_nbf> and C<verify_iat>. Default is C<0>.
 
 =item ignore_claims
 
@@ -1056,12 +1059,6 @@ Set 'nbf' claim (Not Before) to current time + C<relative_nbf> value (in seconds
 
 NOTE: claims are part of the payload and can be used only if the payload is a HASH ref!
 
-=item auto_typ
-
-C<1> - set 'typ' header to value C<JWS> for JWS tokens and C<JWE> for JWE tokens.
-
-C<0> (default) - do not set 'typ' header automatically (you can still set it via C<extra_headers>).
-
 =back
 
 =head1 SEE ALSO
